import logging
import note

FINGER_THUMB    = 0
FINGER_INDEX    = 1 
FINGER_MIDDLE   = 2
FINGER_RING     = 3
FINGER_PINKY    = 4


class FretboardString(object):
    def __init__(self, fretboard, tuning):
        self.fretboard = fretboard
        self.tuning = tuning

    def __unicode__(self):
        return u'%s string' % self.tuning

    def get_note(self, fret=0):
        return self.tuning + fret

    def get_fret(self, note=note.C):
        return note.semitone_distance - self.tuning.semitone_distance

class FretboardNoteDisplay(object):
    def __init__(self, note, specific=False, string=None, color='yellow'):
        self.note = note
        self.specific = specific
        self.string = string
        self.color = color

    def __unicode__(self):
        return u'Display: %s' % self.note

class Fretboard(object):
    def __init__(self, tuning, name=u'Unknown Fretboard', num_frets=14):
        self.string_list = []
        self.note_display_list = []
        self.tuning = tuning
        self.name = name
        self.num_frets = num_frets
        for note in self.tuning:
            self.string_list.append(FretboardString(self, note))    

    def __unicode__(self):
        return unicode(self.name)

    def __hash__(self):
        return hash(self.name)

    def __cmp__(self, right):
        if hasattr(right, 'tuning'):
            return cmp(self.tuning, right.tuning)
        return cmp(id(self), id(right))

    def clear(self):
        self.note_display_list = []
    
    def add_note_display(self, note, **kwargs):
        if isinstance(note, FretboardNoteDisplay):
            note_display = note
        else:
            note_display = FretboardNoteDisplay(note, **kwargs)
        logging.debug(u'%s.add_note_display(%s)' % (self, note_display))
        self.note_display_list.append(note_display)
        return note_display

    def remove_note_display(self, note_display):
        if note_display in self.note_display_list:
            self.note_display_list.remove(note_display)
        return note_display

    def display_interval_list(self, interval_list, clear=True):
        if not interval_list:
            return
        if clear:
            self.clear()
        for note in interval_list.note_list:
            if note.pitch_class == interval_list.root.pitch_class:
                self.add_note_display(FretboardNoteDisplay(note, color='red'))
            else:
                self.add_note_display(FretboardNoteDisplay(note))

    def display_scale(self, scale, clear=True):
        return self.display_interval_list(scale, clear=clear)

    def display_chord(self, chord, clear=True):
        return self.display_interval_list(chord, clear=clear)

    def get_display_list(self, string, position):
        display_list = []
        for note_display in self.note_display_list:
            if string.get_note(position).pitch_class == note_display.note.pitch_class:
                if not note_display.string or note_display.string == string:
                    display_list.append(note_display)
        return display_list

    def get_ascii_diagram(self, vertical=False):
        DISPLAY_CHARS = {'red': 'X'}
        longest_note_name_len = 0
        for string in self.string_list:
            length = len(unicode(string.tuning))
            if length > longest_note_name_len:
                longest_note_name_len = length
            
        diagram = ''
        if vertical:
            for position in range(0, self.num_frets + 1):
                for string in self.string_list:
                    diagram += ' | '
                diagram += '\n'

                for string in self.string_list:
                    logging.debug(u'drawing %s postion %s' % (string, position))
                    display_list = self.get_display_list(string, position)
                    if display_list:
                        display_char = DISPLAY_CHARS.get(
                                display_list[-1].color, u'O')
                        diagram += u' %s ' % display_char
                    else:
                        diagram += u' | '
                diagram += '\n'

                for string in self.string_list:
                    diagram += '_|_'  
                diagram += '\n'
        else:
            BLANK_LINE = (  u'|' + (longest_note_name_len * u' ') + u'   ' + 
                            (u'   |' * (self.num_frets + 1))    ) + u'\n'
            TOP_LINE = (len(BLANK_LINE) - 1) * u'_' + u'\n'
            BOT_LINE = BLANK_LINE.replace(u' ', u'_')
            diagram += TOP_LINE
            string_list = list(self.string_list)
            string_list.reverse()
            for string in string_list:
                note_name = unicode(string.tuning)
                while len(note_name) < longest_note_name_len:
                    note_name = u' ' + note_name    
                diagram += BLANK_LINE
                diagram += u'| %s  ' % note_name 

                for position in range(0, self.num_frets + 1):
                    display_list = self.get_display_list(string, position)
                    if display_list:
                        display_char = DISPLAY_CHARS.get(
                                display_list[-1].color, u'O')
                        diagram += u'-%s-|' % display_char
                    else:
                        diagram += u'---|'
                diagram += u'\n' 
            diagram += BOT_LINE
            diagram += (longest_note_name_len * u' ') + u'     '
            for position in range(0, self.num_frets + 1):
                position_lbl = unicode(position)
                while len(position_lbl) < 2:
                    position_lbl = u' ' + position_lbl     
                diagram += position_lbl + u'  '

        return diagram
    ascii_diagram = property(get_ascii_diagram)


class FingerState(object):
    def __init__(self, type, position_offset, mute_strings=[], fret_strings=[]):
        self.type = type
        self.position_offset = position_offset
        self.mute_strings = list(mute_strings)
        self.fret_strings = list(fret_strings)


class Fingering(object):
    def __init__(self, position=0, fingers=[]):
        self.fingers = []
        self.position = position
        for finger_state in fingers:
            self.add_finger(finger_state)

    def add_finger(self, finger_state):
        for finger in list(self.fingers):
            if finger.type == finger_state.type:
                self.remove_finger(finger)
        self.fingers.append(finger_state)
        return finger_state

    def remove_finger(self, finger_state):
        if finger_state in self.fingers:
            self.fingers.remove(finger_state)
        return finger_state

# Tunings gathered from: http://en.wikipedia.org/wiki/Guitar_tunings

STANDARD_GUITAR = Fretboard((note.E, note.A, note.D, note.G, note.B, note.E),
                            '6-String Guitar - Standard')
DROP_D_GUITAR = Fretboard((note.D, note.A, note.D, note.G, note.B, note.E),
                            '6-String Guitar - Drop D')
DROP_C_GUITAR = Fretboard((note.C, note.G, note.C, note.F, note.A, note.D),
                            '6-String Guitar - Drop C')
DROP_B_1_GUITAR = Fretboard((note.B, note.F_SHARP, note.B, note.E, note.G_SHARP, note.C_SHARP),
                            '6-String Guitar - Drop B(1)')
DROP_B_2_GUITAR = Fretboard((note.B, note.G_FLAT, note.B, note.E, note.A, note.D),
                            '6-String Guitar - Drop B(2)')
DROP_B_3_GUITAR = Fretboard((note.B_FLAT, note.A_FLAT, 
                             note.D_FLAT, note.G_FLAT, 
                             note.B_FLAT, note.E_FLAT),
                            '6-String Guitar - Drop B(3)')
DROP_A_1_GUITAR = Fretboard((note.A, note.E, note.A, note.D, note.F_SHARP, note.B),
                            '6-String Guitar - Drop A(1)')
DROP_A_2_GUITAR = Fretboard((note.A, note.A, note.D, note.G, note.B, note.E),
                            '6-String Guitar - Drop A(2)')
E_FLAT_GUITAR = Fretboard((note.E_FLAT, note.A_FLAT, 
                           note.D_FLAT, note.G_FLAT, 
                           note.B_FLAT, note.E_FLAT),
                            '6-String Guitar - Eb')
D_GUITAR = Fretboard((note.D, note.G, note.C, note.F, note.A, note.D),
                            '6-String Guitar - D')
C_GUITAR = Fretboard((note.C, note.F, note.B_FLAT, note.E_FLAT, note.G, note.C),
                            '6-String Guitar - D')
OPEN_A_1_GUITAR = Fretboard((note.E, note.A, note.C_SHARP, note.E, note.A, note.E),
                            '6-String Guitar - Open A(1)')
OPEN_A_2_GUITAR = Fretboard((note.E, note.A, note.C_SHARP, note.E, note.A, note.C_SHARP),
                            '6-String Guitar - Open A(2)')
OPEN_A_SLIDE_GUITAR = Fretboard((note.E, note.A, note.E, note.A, note.C_SHARP, note.E),
                            '6-String Guitar - Open A Slide')
OPEN_C_GUITAR = Fretboard((note.C, note.G, note.C, note.G, note.C, note.E),
                            '6-String Guitar - Open C')
OPEN_D_1_GUITAR = Fretboard((note.D, note.A, note.D, note.F_SHARP, note.A, note.D),
                            '6-String Guitar - Open D(1)')
OPEN_D_2_GUITAR = Fretboard((note.D, note.A, note.D, note.A, note.D, note.D),
                            '6-String Guitar - Open D(2)')
OPEN_D_MINOR_GUITAR = Fretboard((note.D, note.A, note.D, note.F, note.A, note.D),
                            '6-String Guitar - Open Dm')
OPEN_E_GUITAR = Fretboard((note.E, note.B, note.E, note.G_SHARP, note.B, note.E),
                            '6-String Guitar - Open E')
OPEN_F_GUITAR = Fretboard((note.E, note.B, note.E, note.G_SHARP, note.B, note.E),
                            '6-String Guitar - Open F')
OPEN_G_GUITAR = Fretboard((note.D, note.G, note.D, note.G, note.B, note.D),
                            '6-String Guitar - Open G')
CROSSNOTE_A_GUITAR = Fretboard((note.E, note.A, note.E, note.A, note.C, note.E),
                            '6-String Guitar - Crossnote A')
CROSSNOTE_C_GUITAR = Fretboard((note.C, note.G, note.C, note.G, note.C, note.E_FLAT),
                            '6-String Guitar - Crossnote C')
CROSSNOTE_E_GUITAR = Fretboard((note.E, note.B, note.E, note.G, note.B, note.E),
                            '6-String Guitar - Crossnote E')
CROSSNOTE_G_GUITAR = Fretboard((note.D, note.G, note.D, note.G, note.B_FLAT, note.D),
                            '6-String Guitar - Crossnote G')
RUSSION_SEVEN_STRING_GUITAR = Fretboard((note.D, note.G, note.B, note.D, note.G, note.B, note.D),
                            'Russian 7-String Guitar - Open G')

OPEN_G_DOBRO = Fretboard((note.D, note.G, note.D, note.G, note.B, note.D),
                            '6-String Dobro - Open G')



STANDARD_FOUR_STRING_BASS = Fretboard((note.E, note.A, note.D, note.G),
                            '4-String Bass Guitar - Standard')

FRETBOARDS = [
    STANDARD_GUITAR,
    DROP_D_GUITAR,
    DROP_C_GUITAR,
    DROP_B_1_GUITAR,
    DROP_B_2_GUITAR,
    DROP_B_3_GUITAR,
    DROP_A_1_GUITAR,
    DROP_A_2_GUITAR,
    E_FLAT_GUITAR,
    D_GUITAR,
    C_GUITAR,
    OPEN_A_1_GUITAR,
    OPEN_A_2_GUITAR,
    OPEN_A_SLIDE_GUITAR,
    OPEN_C_GUITAR,
    OPEN_D_1_GUITAR,
    OPEN_D_2_GUITAR,
    OPEN_D_MINOR_GUITAR,
    OPEN_E_GUITAR,
    OPEN_F_GUITAR,
    OPEN_G_GUITAR,
    CROSSNOTE_A_GUITAR,
    CROSSNOTE_C_GUITAR,
    CROSSNOTE_E_GUITAR,
    CROSSNOTE_G_GUITAR,
    STANDARD_FOUR_STRING_BASS,
    OPEN_G_DOBRO
]

if __name__ == '__main__':
    import sys
    from scale import Scale, MAJOR
    from note import C

    scale = Scale(MAJOR, C)

    STANDARD_GUITAR.display_scale(scale)
    print STANDARD_GUITAR.ascii_diagram
