import logging


from PySide.QtGui import QListWidgetItem
class HistoryListItem(QListWidgetItem):
    def __init__(self, history_item, parent):
        self.history_item = history_item
        super(HistoryListItem, self).__init__(unicode(history_item), parent)


from ui.qt4.controllers import MainWindow
from ui.qt4.views.fretboard import FretboardView

class FretboardWindow(MainWindow):
    view_class = FretboardView

    def __init__(self, app, **kwargs):
        from music_theory.scale import MAJOR as MAJOR_SCALE, Scale
        from music_theory.chord import MAJOR as MAJOR_CHORD, Chord
        from music_theory.note import C
        super(FretboardWindow, self).__init__(app, **kwargs)
        self.setWindowTitle(u'PyFret')
        view = self.view_class(self)
        self.display_history = []
        self.render(view)
        self.connect_signals()
        self.init_menus()
        self.cur_interval_list = None
        self.current_scale = None
        self.current_chord = None
        self.view.scale_chooser.set_scale(Scale(MAJOR_SCALE, C))
        self.draw_selected_scale()
        #self.on_selected_chord_change()
        #self.on_selected_scale_change()

    def connect_signals(self):
        from PySide.QtCore import QObject, SIGNAL
        QObject.connect(self.view.scale_chooser, SIGNAL('selectedScaleChanged'),
                self.on_selected_scale_change)
        QObject.connect(self.view.chord_chooser, SIGNAL('selectedChordChanged'),
                self.on_selected_chord_change)
        QObject.connect(self.view.chord_draw_btn, SIGNAL('clicked()'),
                self.draw_selected_chord)
        QObject.connect(self.view.scale_draw_btn, SIGNAL('clicked()'),
                self.draw_selected_scale)
        QObject.connect(self.view.fretboard_chooser, SIGNAL('currentIndexChanged(int)'),
                self.on_fretboard_change)
        QObject.connect(self.view.display_tab_widget, SIGNAL('currentChanged(int)'),
                self.on_display_tab_change)
        QObject.connect(self.view.display_history_lst, SIGNAL('currentItemChanged(QListWidgetItem*,QListWidgetItem*)'),
                self.on_history_item_change)
        QObject.connect(self.view.scale_similarity_table, SIGNAL('itemClicked(QTableWidgetItem*)'),
                self.on_scale_similarity_click)
        QObject.connect(self.view.chord_similarity_table, SIGNAL('itemClicked(QTableWidgetItem*)'),
                self.on_chord_similarity_click)
        QObject.connect(self.view.clear_fretboard_btn, SIGNAL('clicked()'),
                self.clear_display)
        QObject.connect(self.view.display_history_clear_btn, SIGNAL('clicked()'),
                self.clear_display_history)
        QObject.connect(self.view.display_history_remove_btn, SIGNAL('clicked()'),
                self.display_history_remove)
        QObject.connect(self.view.zoom_in_btn, SIGNAL('clicked()'),
                self.zoom_in)
        QObject.connect(self.view.zoom_out_btn, SIGNAL('clicked()'),
                self.zoom_out)

    def init_menus(self):
        self.file_menu = self.menuBar().addMenu(u'File')
        self.file_menu.addAction(
                self.app.ui_actions['quit'])
        self.edit_menu = self.menuBar().addMenu(u'Edit')
        self.edit_menu.addAction(
                self.app.ui_actions['show_scale_type_edit_dialog'])
        self.edit_menu.addAction(
                self.app.ui_actions['show_chord_type_edit_dialog'])
        self.edit_menu.addAction(
                self.app.ui_actions['show_fretboard_edit_dialog'])

    def init_widget_choices(self):
        for widget in ( self.view.scale_chooser, 
                        self.view.chord_chooser,
                        self.view.fretboard_chooser ):
            widget.init_choices()

    def clear_display(self):
        self.view.fretboard_display.fretboard.clear()
        self.view.fretboard_display.refresh()
        self.resize(self.minimumSize())

    def zoom_in(self, **kwargs):
        self.view.fretboard_display.zoom_in()
        self.resize(self.minimumSizeHint())

    def zoom_out(self, **kwargs):
        self.view.fretboard_display.zoom_out()
        self.resize(self.minimumSizeHint())

    def clear_display_history(self):
        while self.view.display_history_lst.count():
            self.view.display_history_lst.takeItem(0)

    def display_history_add(self, interval_list):
        if not interval_list in self.display_history:
            self.display_history.append(interval_list)
            item = HistoryListItem(interval_list, self.view.display_history_lst)
            self.view.display_history_lst.addItem(item)
            self.view.display_history_lst.setCurrentItem(item)

    def display_history_remove(self):
        self.view.display_history_lst.takeItem(
                self.view.display_history_lst.currentRow())

    def display_interval_list(self, interval_list, force=False):
        from PySide.QtCore import Qt
        from music_theory.chord import Chord
        from music_theory.scale import Scale

        if self.cur_interval_list != interval_list or force:
            self.app.setOverrideCursor(Qt.WaitCursor)
            self.cur_interval_list = interval_list
            if isinstance(interval_list, Chord):
                self.current_chord = interval_list
            if isinstance(interval_list, Scale):
                self.current_scale = interval_list
            self.app.processEvents()
            if self.view.fretboard_display.fretboard:
                self.view.fretboard_display.fretboard.display_interval_list(
                        interval_list)
                self.view.fretboard_display.refresh()
            #self.view.display_information_txt.setPlainText(
            #        interval_list.description)
            self.view.display_information_interval_list.interval_list = interval_list
            self.on_display_tab_change(self.view.display_tab_widget.currentIndex())
            self.display_history_add(interval_list)
            self.app.restoreOverrideCursor()

    def on_display_tab_change(self, new_index):
        from PySide.QtCore import Qt, QObject, SIGNAL
        from ui.qt4.views.fretboard import DISPLAY_SCALE_SIMILARITY_TAB_IDX
        from ui.qt4.views.fretboard import DISPLAY_CHORD_SIMILARITY_TAB_IDX
        # Only calculate similarities when tab is visible
        self.app.setOverrideCursor(Qt.WaitCursor)
        self.app.processEvents()
        if new_index == DISPLAY_SCALE_SIMILARITY_TAB_IDX:
            from ui.qt4.widgets.scale_similarity_table import ScaleSimilarityTable
            # XXX: This removal and readdition of the widget shouldn't be
            #      Necessary, but without it, some fields show blank on 
            #      Subsequent redraws :(
            old_table = self.view.scale_similarity_table 
            self.view.scale_similarity_layout.removeWidget(old_table)

            new_table = ScaleSimilarityTable()
            self.view.scale_similarity_layout.addWidget(new_table)
            self.view.scale_similarity_table = new_table
            old_table.deleteLater()
            new_table.target = self.cur_interval_list
            QObject.connect(new_table, SIGNAL('itemClicked(QTableWidgetItem*)'),
                    self.on_scale_similarity_click)
        elif new_index == DISPLAY_CHORD_SIMILARITY_TAB_IDX:
            from ui.qt4.widgets.chord_similarity_table import ChordSimilarityTable
            # XXX: This removal and readdition of the widget shouldn't be
            #      Necessary, but without it, some fields show blank on 
            #      Subsequent redraws :(
            old_table = self.view.chord_similarity_table 
            self.view.chord_similarity_layout.removeWidget(old_table)

            new_table = ChordSimilarityTable()
            self.view.chord_similarity_layout.addWidget(new_table)
            self.view.chord_similarity_table = new_table
            old_table.deleteLater()
            new_table.target = self.cur_interval_list
            QObject.connect(new_table, SIGNAL('itemClicked(QTableWidgetItem*)'),
                    self.on_chord_similarity_click)
        self.app.restoreOverrideCursor()

    

    def draw_selected_scale(self, **kwargs):
        self.display_interval_list(self.current_scale)
        return self.current_scale
        
    def on_selected_scale_change(self, **kwargs):
        self.current_scale = self.view.scale_chooser.scale

    def draw_selected_chord(self, **kwargs):
        self.display_interval_list(self.current_chord)
        return self.current_chord

    def on_selected_chord_change(self, **kwargs):
        self.current_chord = self.view.chord_chooser.chord

    def on_fretboard_change(self, **kwargs):
        fretboard = self.view.fretboard_chooser.fretboard
        self.view.fretboard_display.fretboard = fretboard
        #self.on_selected_scale_change()
        self.display_interval_list(self.cur_interval_list, force=True)

    def on_history_item_change(self, new_item, old_item):
        if new_item:
            self.display_interval_list(new_item.history_item)

    def on_scale_similarity_click(self, clicked_item):
        if clicked_item:
            self.display_interval_list(clicked_item.scale_item)

    def on_chord_similarity_click(self, clicked_item):
        if clicked_item:
            self.display_interval_list(clicked_item.chord_item)

